@@ -1,24 +1,11 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
  * Copyright (c) 2004-2005 Silicon Graphics, Inc.
  * All Rights Reserved.
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation.
- *
- * This program is distributed in the hope that it would be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write the Free Software Foundation,
- * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <xfs/xfs.h>
-#include <xfs/command.h>
-#include <xfs/input.h>
+#include "command.h"
+#include "input.h"
 #include <sys/sendfile.h>
 #include "init.h"
 #include "io.h"
@@ -57,11 +44,11 @@ send_buffer(
 
 	*total = 0;
 	while (count > 0) {
-		bytes = sendfile64(file->fd, fd, &off, bytes_remaining);
+		bytes = sendfile(file->fd, fd, &off, bytes_remaining);
 		if (bytes == 0)
 			break;
 		if (bytes < 0) {
-			perror("sendfile64");
+			perror("sendfile");
 			return -1;
 		}
 		ops++;
@@ -82,17 +69,20 @@ sendfile_f(
 	long long	count, total;
 	size_t		blocksize, sectsize;
 	struct timeval	t1, t2;
-	char		s1[64], s2[64], ts[64];
 	char		*infile = NULL;
-	int		Cflag;
+	int		Cflag, qflag;
 	int		c, fd = -1;
 
-	Cflag = 0;
+	Cflag = qflag = 0;
 	init_cvtnum(&blocksize, &sectsize);
-	while ((c = getopt(argc, argv, "Cf:i:")) != EOF) {
+	while ((c = getopt(argc, argv, "Cf:i:q")) != EOF) {
 		switch (c) {
 		case 'C':
 			Cflag = 1;
+			break;
+		case 'q':
+			qflag = 1;
+			break;
 		case 'f':
 			fd = atoi(argv[1]);
 			if (fd < 0 || fd >= filecount) {
@@ -113,7 +103,7 @@ sendfile_f(
 
 	if (!infile)
 		fd = filetable[fd].fd;
-	else if ((fd = openfile(infile, NULL, IO_READONLY, 0)) < 0)
+	else if ((fd = openfile(infile, NULL, IO_READONLY, 0, NULL)) < 0)
 		return 0;
 
 	if (optind == argc - 2) {
@@ -131,10 +121,10 @@ sendfile_f(
 			goto done;
 		}
 	} else {
-		struct stat64	stat;
+		struct stat	stat;
 
-		if (fstat64(fd, &stat) < 0) {
-			perror("fstat64");
+		if (fstat(fd, &stat) < 0) {
+			perror("fstat");
 			goto done;
 		}
 		count = stat.st_size;
@@ -144,23 +134,12 @@ sendfile_f(
 	c = send_buffer(offset, count, fd, &total);
 	if (c < 0)
 		goto done;
+	if (qflag)
+		goto done;
 	gettimeofday(&t2, NULL);
 	t2 = tsub(t2, t1);
 
-	/* Finally, report back -- -C gives a parsable format */
-	timestr(&t2, ts, sizeof(ts), Cflag ? VERBOSE_FIXED_TIME : 0);
-	if (!Cflag) {
-		cvtstr((double)total, s1, sizeof(s1));
-		cvtstr(tdiv((double)total, t2), s2, sizeof(s2));
-		printf(_("sent %lld/%lld bytes from offset %lld\n"),
-			total, count, (long long)offset);
-		printf(_("%s, %d ops; %s (%s/sec and %.4f ops/sec)\n"),
-			s1, c, ts, s2, tdiv((double)c, t2));
-	} else {/* bytes,ops,time,bytes/sec,ops/sec */
-		printf("%lld,%d,%s,%.3f,%.3f\n",
-			total, c, ts,
-			tdiv((double)total, t2), tdiv((double)c, t2));
-	}
+	report_io_times("sent", &t2, (long long)offset, count, total, c, Cflag);
 done:
 	if (infile)
 		close(fd);
@@ -170,8 +149,8 @@ done:
 void
 sendfile_init(void)
 {
-	sendfile_cmd.name = _("sendfile");
-	sendfile_cmd.altname = _("send");
+	sendfile_cmd.name = "sendfile";
+	sendfile_cmd.altname = "send";
 	sendfile_cmd.cfunc = sendfile_f;
 	sendfile_cmd.argmin = 2;
 	sendfile_cmd.argmax = -1;
