@@ -5,6 +5,7 @@
  */
 
 #include "libxfs_priv.h"
+#include "libxfs.h"
 #include "libxfs_io.h"
 #include "init.h"
 #include "xfs_fs.h"
@@ -230,7 +231,7 @@ xfs_flags2diflags2(
  * This was once shared with the kernel, but has diverged to the point
  * where it's no longer worth the hassle of maintaining common code.
  */
-int
+static int
 libxfs_ialloc(
 	xfs_trans_t	*tp,
 	xfs_inode_t	*pip,
@@ -354,7 +355,7 @@ libxfs_ialloc(
 	case S_IFLNK:
 		ip->i_d.di_format = XFS_DINODE_FMT_EXTENTS;
 		ip->i_df.if_flags = XFS_IFEXTENTS;
-		ip->i_df.if_bytes = ip->i_df.if_real_bytes = 0;
+		ip->i_df.if_bytes = 0;
 		ip->i_df.if_u1.if_root = NULL;
 		break;
 	default:
@@ -380,69 +381,6 @@ libxfs_ialloc(
 	return 0;
 }
 
-void
-libxfs_iprint(
-	xfs_inode_t		*ip)
-{
-	struct xfs_icdinode	*dip;
-	xfs_extnum_t		i = 0;
-	xfs_ifork_t		*ifp;		/* inode fork pointer */
-	struct xfs_iext_cursor	icur;
-	xfs_bmbt_irec_t		rec;
-
-	printf("Inode %lx\n", (unsigned long)ip);
-	printf("    i_ino %llx\n", (unsigned long long)ip->i_ino);
-
-	if (ip->i_df.if_flags & XFS_IFEXTENTS)
-		printf("EXTENTS ");
-	printf("\n");
-	printf("    i_df.if_bytes %d\n", ip->i_df.if_bytes);
-	printf("    i_df.if_u1.if_root/if_data %lx\n",
-		(unsigned long)ip->i_df.if_u1.if_root);
-	if (ip->i_df.if_flags & XFS_IFEXTENTS) {
-		ifp = XFS_IFORK_PTR(ip, XFS_DATA_FORK);
-		for_each_xfs_iext(ifp, &icur, &rec) {
-			printf("\t%d: startoff %llu, startblock 0x%llx,"
-				" blockcount %llu, state %d\n",
-				i, (unsigned long long)rec.br_startoff,
-				(unsigned long long)rec.br_startblock,
-				(unsigned long long)rec.br_blockcount,
-				(int)rec.br_state);
-			i++;
-		}
-	}
-	printf("    i_df.if_broot %lx\n", (unsigned long)ip->i_df.if_broot);
-	printf("    i_df.if_broot_bytes %x\n", ip->i_df.if_broot_bytes);
-
-	dip = &ip->i_d;
-	printf("\nOn disk portion\n");
-	printf("    di_mode %o\n", VFS_I(ip)->i_mode);
-	printf("    di_version %x\n", (uint)dip->di_version);
-	switch (ip->i_d.di_format) {
-	case XFS_DINODE_FMT_LOCAL:
-		printf("    Inline inode\n");
-		break;
-	case XFS_DINODE_FMT_EXTENTS:
-		printf("    Extents inode\n");
-		break;
-	case XFS_DINODE_FMT_BTREE:
-		printf("    B-tree inode\n");
-		break;
-	default:
-		printf("    Other inode\n");
-		break;
-	}
-	printf("   di_nlink %x\n", VFS_I(ip)->i_nlink);
-	printf("   di_uid %d\n", dip->di_uid);
-	printf("   di_gid %d\n", dip->di_gid);
-	printf("   di_nextents %d\n", dip->di_nextents);
-	printf("   di_size %llu\n", (unsigned long long)dip->di_size);
-	printf("   di_gen %x\n", VFS_I(ip)->i_generation);
-	printf("   di_extsize %d\n", dip->di_extsize);
-	printf("   di_flags %x\n", dip->di_flags);
-	printf("   di_nblocks %llu\n", (unsigned long long)dip->di_nblocks);
-}
-
 /*
  * Writes a modified inode's changes out to the inode's on disk home.
  * Originally based on xfs_iflush_int() from xfs_inode.c in the kernel.
@@ -543,8 +481,6 @@ libxfs_alloc_file_space(
 	xfs_filblks_t	datablocks;
 	xfs_filblks_t	allocated_fsb;
 	xfs_filblks_t	allocatesize_fsb;
-	xfs_fsblock_t	firstfsb;
-	struct xfs_defer_ops free_list;
 	xfs_bmbt_irec_t *imapp;
 	xfs_bmbt_irec_t imaps[1];
 	int		reccount;
@@ -582,10 +518,8 @@ libxfs_alloc_file_space(
 		}
 		xfs_trans_ijoin(tp, ip, 0);
 
-		xfs_defer_init(&free_list, &firstfsb);
 		error = xfs_bmapi_write(tp, ip, startoffset_fsb, allocatesize_fsb,
-				xfs_bmapi_flags, &firstfsb, 0, imapp,
-				&reccount, &free_list);
+				xfs_bmapi_flags, 0, imapp, &reccount);
 
 		if (error)
 			goto error0;
@@ -593,10 +527,6 @@ libxfs_alloc_file_space(
 		/*
 		 * Complete the transaction
 		 */
-		error = xfs_defer_finish(&tp, &free_list);
-		if (error)
-			goto error0;
-
 		error = xfs_trans_commit(tp);
 		if (error)
 			break;
@@ -611,7 +541,6 @@ libxfs_alloc_file_space(
 	return error;
 
 error0:	/* Cancel bmap, cancel trans */
-	xfs_defer_cancel(&free_list);
 	xfs_trans_cancel(tp);
 	return error;
 }
@@ -762,7 +691,7 @@ xfs_inode_verifier_error(
  * repair can validate it against the state of the log.
  */
 xfs_lsn_t	libxfs_max_lsn = 0;
-pthread_mutex_t	libxfs_max_lsn_lock = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t	libxfs_max_lsn_lock = PTHREAD_MUTEX_INITIALIZER;
 
 bool
 xfs_log_check_lsn(
