@@ -1,21 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0+
 /*
  * Copyright (C) 2018 Oracle.  All Rights Reserved.
- *
  * Author: Darrick J. Wong <darrick.wong@oracle.com>
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it would be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write the Free Software Foundation,
- * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 #include "xfs.h"
 #include <pthread.h>
@@ -86,6 +72,15 @@
  * the previous two phases are retried here; if there are uncorrectable
  * errors, xfs_scrub stops here.
  *
+ * To perform the actual repairs (or optimizations), we iterate all the
+ * items on the per-AG action item list and ask the kernel to repair
+ * them.  Items which are successfully repaired are removed from the
+ * list.  If an item is not acted upon successfully (or the kernel asks us
+ * to try again), we retry the actions until there is nothing left to
+ * fix or we fail to make forward progress.  In that event, the
+ * unfinished items are recorded as errors.  If there are no errors at
+ * this point, we call FSTRIM on the filesystem.
+ *
  * The next phase is the "check directory tree" phase.  In this phase,
  * every directory is opened (via file handle) to confirm that each
  * directory is connected to the root.  Directory entries are checked
@@ -427,10 +422,9 @@ run_scrub_phases(
 	bool			moveon = true;
 	unsigned int		debug_phase = 0;
 	unsigned int		phase;
-	unsigned int		nr_threads;
 	int			rshift;
 
-	if (debug && debug_tweak_on("XFS_SCRUB_PHASE"))
+	if (debug_tweak_on("XFS_SCRUB_PHASE"))
 		debug_phase = atoi(getenv("XFS_SCRUB_PHASE"));
 
 	/* Run all phases of the scrub tool. */
@@ -459,12 +453,14 @@ run_scrub_phases(
 		if (!moveon)
 			break;
 		if (sp->estimate_work) {
-			moveon = sp->estimate_work(ctx, &max_work, &nr_threads,
-					&rshift);
+			unsigned int		work_threads;
+
+			moveon = sp->estimate_work(ctx, &max_work,
+					&work_threads, &rshift);
 			if (!moveon)
 				break;
 			moveon = progress_init_phase(ctx, progress_fp, phase,
-					max_work, rshift, nr_threads);
+					max_work, rshift, work_threads);
 		} else {
 			moveon = progress_init_phase(ctx, NULL, phase, 0, 0, 0);
 		}
@@ -491,6 +487,27 @@ _("Scrub aborted after phase %d."),
 	return moveon;
 }
 
+static void
+report_modifications(
+	struct scrub_ctx	*ctx)
+{
+	if (ctx->repairs == 0 && ctx->preens == 0)
+		return;
+
+	if (ctx->repairs && ctx->preens)
+		fprintf(stdout,
+_("%s: repairs made: %llu; optimizations made: %llu.\n"),
+				ctx->mntpoint, ctx->repairs, ctx->preens);
+	else if (ctx->preens == 0)
+		fprintf(stdout,
+_("%s: repairs made: %llu.\n"),
+				ctx->mntpoint, ctx->repairs);
+	else if (ctx->repairs == 0)
+		fprintf(stdout,
+_("%s: optimizations made: %llu.\n"),
+				ctx->mntpoint, ctx->preens);
+}
+
 static void
 report_outcome(
 	struct scrub_ctx	*ctx)
@@ -525,9 +542,16 @@ report_outcome(
 	 * setting up the scrub and we actually saw corruptions.  Warnings
 	 * are not corruptions.
 	 */
-	if (ctx->scrub_setup_succeeded && total_errors > 0)
-		fprintf(stderr, _("%s: Unmount and run xfs_repair.\n"),
-				ctx->mntpoint);
+	if (ctx->scrub_setup_succeeded && total_errors > 0) {
+		char		*msg;
+
+		if (ctx->mode == SCRUB_MODE_DRY_RUN)
+			msg = _("%s: Re-run xfs_scrub without -n.\n");
+		else
+			msg = _("%s: Unmount and run xfs_repair.\n");
+
+		fprintf(stderr, msg, ctx->mntpoint);
+	}
 }
 
 int
@@ -728,6 +752,7 @@ main(
 		ctx.runtime_errors++;
 
 out:
+	report_modifications(&ctx);
 	report_outcome(&ctx);
 
 	if (ctx.errors_found) {
@@ -755,9 +780,15 @@ out:
 	 * for capturing all the log messages if the scrub fails, because the
 	 * fail service uses the service name to gather log messages for the
 	 * error report.
+	 *
+	 * Note: We don't count a lack of kernel support as a service failure
+	 * because we haven't determined that there's anything wrong with the
+	 * filesystem.
 	 */
 	if (is_service) {
 		sleep(2);
+		if (!ctx.scrub_setup_succeeded)
+			return 0;
 		if (ret != SCRUB_RET_SUCCESS)
 			return 1;
 	}
